module FormParser3 (formP) where

import Data.Char
import Form
import Parser

-- The tokens of the syntax for formulas
data Token = AndT | OrT | NotT | LPT | RPT | TT | FT | IdT String
             deriving (Eq,Show)

-- The 'scanner' or 'tokenizer'
formS :: Parser Char [Token]
formS = list (spaces *** formS' >>> snd) *** spaces >>> fst
  where
  spaces = list (item ' ')
  formS' =
    (item '&' >>> const AndT) |||
    (item '|' >>> const OrT) |||
    (item '~' >>> const NotT) |||
    (item '(' >>> const LPT) |||
    (item ')' >>> const RPT) |||
    (item 'T' >>> const TT) |||
    (item 'F' >>> const FT) |||
    (identifier >>> IdT)

formP :: Parser Token Form
formP  =
  form1P *** optional (item OrT *** formP >>> snd) >>> orF
  where orF(f1, Just f2) = f1 :|: f2
        orF(f1, Nothing) = f1

-- form1P does not allow |
form1P :: Parser Token Form
form1P  =
  form2P *** optional (item AndT *** form1P >>> snd) >>> andF
  where andF(f1, Just f2) = f1 :&: f2
        andF(f1, Nothing) = f1

-- form2P does not allow &
form2P :: Parser Token Form
form2P  =
  (item NotT *** form2P >>> (Not . snd)) |||
  (item TT >>> const T) |||
  (item FT >>> const F) |||
  (one isIdT >>> var) |||
  enclose LPT RPT formP
    where
    var(IdT s) = Var s

    isIdT(IdT _) = True
    isIdT _ = False

formSP :: String -> Maybe (Form, [Token])
formSP s = case formS s of
             Just(ss,[]) -> formP ss
             _ -> Nothing

test = formSP " x3 | ((x1 | ~x3) & ~ x2)& ~a &~ ( (  d1  & d2)&~d2&d1 ) "
