module Sol_Exercise_11 where
import Data.Char
import qualified Data.List as List
import Data.Maybe
import Parser
import RegEx
import Test.QuickCheck


{- Library -- nicht veraendern -}

data Html = Text String | Block String [Html]
    deriving (Eq, Show)
data HtmlT = OpenT String | CloseT String | WordT String
    deriving (Eq, Show)

htmlP_ex1 = [WordT "hello", WordT "world"]
htmlP_ex2 = [OpenT "sv", WordT "hej", WordT "vaerld", CloseT "sv"]
htmlP_ex3 = [OpenT "a", CloseT "a", WordT "b"]
htmlP_ex4 = [OpenT "a", WordT "foo", WordT "bar"]
htmlP_ex5 = [OpenT "a", OpenT "b", WordT "c", CloseT "b", WordT "d", CloseT "a"]

vocabs = ["i", "kyoto", "more", "now", "tokyo", "want", "won"]
text1 = ["i", "want", "mroe"]
text2 = ["and", "i", "tnaw", "it", "now"]
text3 =["tokyo", "toyko", "kyoto"]


{- G11.1 -}

safeUnscrambleWord :: [String] -> String -> Either (String, [String]) String
safeUnscrambleWord vocabs word
    | word `elem` vocabs = Right word
    | otherwise = Left (word, filter (\vocab -> List.sort vocab == sword) vocabs)
  where
    sword = List.sort word

safeUnscrambleWords :: [String] -> [String] -> [Either (String, [String]) String]
safeUnscrambleWords = map . safeUnscrambleWord


{- G11.2 -}

-- Siehe FormParser3.hs


{- G11.3 -}

isAlphaEtc :: Char -> Bool
isAlphaEtc c = isAlpha c || c `elem` "_$"

isAlphaNumEtc :: Char -> Bool
isAlphaNumEtc c = isDigit c || isAlphaEtc c

identifier' :: Parser Char String
identifier' =
  list1 (one isAlphaEtc) *** list (one isAlphaNumEtc) >>> uncurry (++)


{- H11.1 -}

readNat :: [Char] -> Int
readNat xs = foldl (\a b -> 10 * a + digitToInt b) 0 xs

nat :: Parser Char Int
nat = list1 (one isDigit) >>> readNat

int :: Parser Char Int
int = (optional (item '-') >>> isJust) *** nat >>> \(neg, n) -> n * if neg then -1 else 1

double :: Parser Char Double
double = ((optional (item '-') >>> isJust)
          *** (nat >>> fromIntegral)
          *** item '.'
          *** postDot)
          >>> \(neg,(pre,(_, post)))-> (if neg then -1 else 1) * (pre + post)
  where
    postDot = list1 (one isDigit)
        >>> \ks -> fromIntegral (readNat ks) / 10**fromIntegral (length ks)


{- H11.2 -}

htmlS :: Parser Char [HtmlT]
htmlS = list1 (space *** (open ||| close ||| text) >>> snd) *** space >>> fst
  where
    tag = enclose '<' '>'
    open = tag identifier >>> OpenT
    close = tag (item '/' *** identifier >>> snd) >>> CloseT
    text = (list1 $ one (\c -> not (isSpace c) && c /= '<' && c /= '>')) >>> WordT
    space = list $ one isSpace

infixr 5 ***=

(***=) :: Parser a b -> (b -> Parser a c) -> Parser a (b,c)
p1 ***= p2 = \xs -> case p1 xs of
    Nothing -> Nothing
    Just (b, ys) -> case p2 b ys of
        Nothing -> Nothing
        Just (c, zs) -> Just ((b,c), zs)

htmlP :: Parser HtmlT Html
htmlP = block ||| text
  where
    open (OpenT id : ts) = Just (id, ts)
    open _ = Nothing
    -- Alternative:
    -- open = one isOpenT >>> openId
    -- isOpenT (OpenT _) = True
    -- isOpenT _ = False
    -- openId (OpenT id) = id

    word (WordT w : ts) = Just (w, ts)
    word _ = Nothing

    text = list1 word >>> Text . unwords

    block xs = ((open ***= \id -> (list htmlP *** item (CloseT id))) >>> \(id, (htmls, _)) -> Block id htmls) xs



{- H11.3 -}

{-WETT-}
anyOne :: Parser a a
anyOne = one (const True)

prefix :: Eq a => a -> Parser a b -> Parser a b
prefix a p = item a *** p >>> snd

escape :: Parser Char Char
escape = prefix '\\' anyOne

subRange :: Parser Char (Char, Char)
subRange =
  (escape ||| one (`notElem` "\\]"))
    *** optional (prefix '-' (escape ||| one (/= '\\')))
  >>> \(from, to) -> (from, fromMaybe from to)

range :: Parser Char [(Char, Char)]
range = enclose '[' ']' (list subRange)

atom :: Parser Char RegEx
atom =
  item '.' >>> const Any
  ||| (escape ||| one (`notElem` ".\\()[]{}*?+|")) >>> (\c -> One [(c, c)])
  ||| range >>> One
  ||| enclose '(' ')' regEx

numRep :: Parser Char (Int, Maybe Int)
numRep =
  nat *** optional (prefix ',' (optional nat))
  >>> \(from, to) -> (from, fromMaybe (Just from) to)

rep :: Parser Char (Int, Maybe Int)
rep =
  item '*' >>> const (0, Nothing)
  ||| item '?' >>> const (0, Just 1)
  ||| item '+' >>> const (1, Nothing)
  ||| enclose '{' '}' numRep

factor :: Parser Char RegEx
factor =
  atom *** list rep
  >>> uncurry (foldl Repeat)

term :: Parser Char RegEx
term =
  factor *** list factor
  >>> uncurry (foldl Concat)

regEx :: Parser Char RegEx
regEx =
  term *** list (prefix '|' term)
  >>> uncurry (foldl Alt)
{-TTEW-}
